#!/bin/bash

##    QUINCY terrestrial biosphere model with coupled carbon, nitrogen, and phosphorus cycles
##    Copyright (C) 2021 Max Planck Institute for Biogeochemistry (MPI-BGC), Jena, Germany 
##    Contributors: @TODO: see file?
##
##    This file is part of the QUINCY model.
##    The QUINCY model is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License version 3,
##    as published by the Free Software Foundation.
##
##    The QUINCY model is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with the QUINCY model.  If not, see <https://www.gnu.org/licenses/>.

# First version   by Julia Nabel   (2021-12-03)
#
# run this script by executing: ./compile_quincy.sh [SETUP]
# 
# this script compiles quincy according to the specified SETUP (update, debug, standard, perform)
# in addition it collects information on the current git status and stores it alongside the binary
#

compile_setup=$1 #specify setup here (one of: update, debug, standard, perform) or pass as argument

########################################################################################################
###### END OF USER INTERFACE #########################################
########################################################################################################
cd "$( dirname "$0" )"
compile_script_location=$(pwd)
currentDate=$(date '+%Y-%m-%d %H:%M:%S')
username=$(whoami)

# specification of default settings / assumed filenames / etc.
default_compile_setup="update"
fallback_compile_setup="debug"
binary_git_info_file="binary_git_info.txt"
binary_git_dat_info="binary_sinfo.txt"
build_directory="./build"

#@todo: check environment / allow for different compilers / whatever wished... 
COMP="x86_64-intel"

#--- start
if [ ! -d ${build_directory} ]; then
  echo " EXIT! build directory not found -- please check your quincy clone."
  exit 1
fi
cd ${build_directory}

#--- try to compile
# get last used compile option
ifoinfo=$(ls -l ./ | grep '>' | awk '{print $11}')

if [[ ${compile_setup} == "" ]]; then
  # if not specified use default
  compile_setup=${default_compile_setup}
  if  [[ ${compile_setup} = "update" ]]; then
    if [[ ${ifoinfo} = "" ]]; then
      # if default is update but there is no info on previous compiling: use fallback
      compile_setup=${fallback_compile_setup}
    fi
  fi
fi

if  [[ ${compile_setup} = "update" ]]; then
  time make -j 4 && time make -j 4
elif [[ ${compile_setup} = "debug" ]]; then
  rm -f Make.inc.local
  echo "Compiling with debug options"
  ln -s Make.inc.local.ifort_debug_BGC Make.inc.local
  make clean
  sync
  time make -j 4
elif [[ ${compile_setup} = "standard" ]]; then
  rm -f Make.inc.local
  echo "Compiling with standard options"
  ln -s Make.inc.local.ifort_BGC Make.inc.local
  make clean
  sync
  time make -j 4
elif [[ ${compile_setup} = "perform" ]]; then
  rm -f Make.inc.local
  echo "Compiling with performance options"
  ln -s Make.inc.local.ifort_perform_BGC Make.inc.local
  make clean
  sync
  time make -j 4
else
  echo " EXIT! unknown compile option specified: ${compile_setup}." 
  echo " Please choose one of update, debug, standard or perform."
  exit 1
fi

#--- collect current git info
# leave build directory
cd ${compile_script_location}

# check if in git repro (exit status == 0)
git log > /dev/null 2>&1
exitGitCN=$?

# try to collect info
if [ "$exitGitCN" -eq 0 ]; then # if exit status is zero, i.e., all good
  gitCommitNum=$(git log | head -n 1 | cut -c 8-15)
  # console:
  echo " "
  echo " The git commit-number of your local git repo and branch which you just compiled is: ""$gitCommitNum"
  echo " (commit info: ""$(git show -s --oneline $gitCommitNum))"
  # info file:
  echo "Compiled on ${currentDate} by User: ${username}" > ${build_directory}/${binary_git_info_file}
  echo "from commit ${gitCommitNum}"      >>   ${build_directory}/${binary_git_info_file}
  echo "$(git status | head -n 1)"        >>   ${build_directory}/${binary_git_info_file}
  echo " "                                >>   ${build_directory}/${binary_git_info_file}
  echo "Commit header :"                  >>   ${build_directory}/${binary_git_info_file}
  echo "$(git show -s --oneline)"         >>   ${build_directory}/${binary_git_info_file}
  echo " "                                >>   ${build_directory}/${binary_git_info_file}
  echo "Changed files (if any):"          >>   ${build_directory}/${binary_git_info_file}
  echo "$(git status -s)"                 >>   ${build_directory}/${binary_git_info_file}
  echo " "                                >>   ${build_directory}/${binary_git_info_file}
  echo " "                                >>   ${build_directory}/${binary_git_info_file}
  echo "Detailed commit message:"         >>   ${build_directory}/${binary_git_info_file}
  echo "$(git show -s $gitCommitNum)"     >>   ${build_directory}/${binary_git_info_file}
  echo " "                                >>   ${build_directory}/${binary_git_info_file}
  echo "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" >>   ${build_directory}/${binary_git_info_file}
  echo "Detailed status:"                 >>   ${build_directory}/${binary_git_info_file}
  echo "$(git status)"                    >>   ${build_directory}/${binary_git_info_file}
  echo " "                                >>   ${build_directory}/${binary_git_info_file}
  echo "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" >>   ${build_directory}/${binary_git_info_file}
  echo "Diff:"                            >>   ${build_directory}/${binary_git_info_file}
  echo "$(git diff)"                      >>   ${build_directory}/${binary_git_info_file}

  # derive some more info for plotting
  gitBranch=$(git status | head -n 1 | cut -c 13- )
  gitShortStatusFirstLetters=$(git status -s | head -n 1 | cut -c 1-2)
  if [ $gitShortStatusFirstLetters == "M" ]; then
    gitStatus="pending_commit"
  elif [ $gitShortStatusFirstLetters == " M" ]; then
    gitStatus="modified"
  else
    gitStatus="unchanged"
  fi
  echo "commit branch status" > ${build_directory}/${binary_git_dat_info}
  echo "${gitCommitNum} ${gitBranch} ${gitStatus}" >> ${build_directory}/${binary_git_dat_info}

  mv  ${build_directory}/${binary_git_info_file} ${COMP}/${binary_git_info_file}
  mv  ${build_directory}/${binary_git_dat_info} ${COMP}/${binary_git_dat_info}
else
  echo "The commit number of your local git repo which you just attempted to compile could not be identified, please check!"
  exit 1
fi
