#!/usr/bin/env Rscript

##    QUINCY terrestrial biosphere model with coupled carbon, nitrogen, and phosphorus cycles
##    Copyright (C) 2019 Max Planck Institute for Biogeochemistry (MPI-BGC), Jena, Germany 
##    Contributors: Caldararu S., Engel J., Kern M., Pallandt M., Schnur R., Thum T., Yu L., Zaehle S.
##
##    This file is part of the QUINCY model.
##    The QUINCY model is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License version 3,
##    as published by the Free Software Foundation.
##
##    The QUINCY model is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with the QUINCY model.  If not, see <https://www.gnu.org/licenses/>.

#########################################################################################
##
## generate > transient < met forcing for Fluxnet sites using the PLUMBER2 model intercomparison project dataset
## note! 
##    the input files used here were provided by the PLUMBER2 project through https://modelevaluation.org/
##    * these files contain leap years ! which are 1992 1996 2000 2004 ... each 4 years with the February 29th added (see section "remove leap days from meteo data")
##
## run this script by:
##  cd contrib/forcing_scripts/
##  Rscript --vanilla ./generate_plumber2_transient_quincy_forcing.R
## 
## note: allsiteinfo is ./generate_plumber2_site_list.dat
## 
## output will be generated by default in:
##  ./new_plumber2_transient_met_forcing/
## 
#########################################################################################


## library
library(ncdf4)

## seed
set.seed(123)

## set in/out path & create out folder
ipath <- "/Net/Groups/BGI/work_2/QUINCY/SLM/Data/PLUMBER2_siteset_data/"
opath <- "./new_plumber2_transient_met_forcing/"
dir.create(opath, showWarnings=T, recursive=T)

## input file pattern definitions
infile_ext='_Met.nc'

## read site info file
allsiteinfo       <- read.table("generate_plumber2_site_list.dat", sep="", header=T, stringsAsFactor=F)
allsitename       <- allsiteinfo$site_id
allsitestartyear  <- allsiteinfo$start
allsiteendyear    <- allsiteinfo$end
allsiteorigin     <- allsiteinfo$origin      # refers to the flux tower network and version that the data is from
site_list         <- allsitename
allsiteDataYears  <- allsiteendyear - allsitestartyear + 1

## create array with random years of the site-forcing for 1901-2016 (115 yrs) per site, to be placed in front of the actual forcing data
## this range of data is not needed for all sites, but the latest year data of plumber2 sites starts (is available) is 2016 (see generate_plumber2_site_list.dat)
periodForcingYears         <- 115
randForcingYears           <- array(NA,dim=c(periodForcingYears, length(site_list)))
colnames(randForcingYears) <- site_list
for (s in 1:length(site_list)){
  randForcingYears[,s] <- sample(1:allsiteDataYears[s], periodForcingYears, replace=T)
}

## create day of year and hour of day and dpm (days per month) default values for output met_forcing files (without leap years)
doy <- ceiling(seq(1/48,365,1/48))
hod <- rep(seq(0,23.5,0.5),365)
dpm <- c(31,28,31,30,31,30,31,31,30,31,30,31)  

## define output-file headers
header1 <- c('year','doy','hour','swvis_srf_down','lw_srf_down','t_air','q_air','press_srf',
             'rain','snow','wind_air','co2_mixing_ratio','co2_dC13','co2DC14','nhx_srf_down','noy_srf_down','p_srf_down')
header2 <- c('-','-','-','Wm-2','Wm-2','K','g/kg','hPa','mm/day','mm/day','m/s','ppm','per-mill','per-mill','mg/m2/day','mg/m2/day','mg/m2/day')

## CO2 info files
co2y      <- read.table('/Net/Groups/BGI/model/OCN/input/gridded/CO2/GCP2019_co2_global.dat')
co2y_dC13 <- read.table('/Net/Groups/BGI/work_2/QUINCY/SLM/InputDataSources/Atm/delta13C_in_air_input4MIPs_GM_1850-2018_extrapolated.txt')
co2y_DC14 <- read.table('/Net/Groups/BGI/work_2/QUINCY/SLM/InputDataSources/Atm/Delta14C_in_air_input4MIPs_SHTRNH_1850-2018_extrapolated.txt')

## forcing will be created for years 1901 up to 2018 (differs across sites)
forcing_startyear   <- 1901
forcing_endyear     <- max(allsiteendyear)  

## loop over all sites in the site_list
for (s in 1:length(site_list)) {
#for (s in 1:1) {   # for debugging
  print(site_list[s])
  
  # get the (randomly selected) year of the actual measured forcing for the "transient" forcing period before measured forcing is available
  pickyear <- randForcingYears[,s]

  # start/end year and length of actual measured data
  startyear  <- allsitestartyear[s]
  endyear    <- allsiteendyear[s]
  nyear      <- allsiteDataYears[s]
  siteorigin <- allsiteorigin[s]
  
  # check if a site-specific netcdf file exists, e.g., DE-Hai_2000-2012_FLUXNET2015_Met.nc
  sDataFile <- paste0(ipath, site_list[s], '_', startyear, '-', endyear, '_', siteorigin, infile_ext)
  existF    <- file.exists(sDataFile)
  
  # define output file, e.g., DE-Hai_t_1901-2012.dat  (the '_t_' denotes transient forcing files)
  outname   <- paste0(opath, site_list[s], "_t_", forcing_startyear, "-", endyear, ".dat")

  # if data for this site exists
  if(existF) {
    # replace old output file
    if (file.exists(outname)) file.remove(outname) 
    # write header, space separated
    write.table(t(cbind(header1,header2)), file=outname,
                append=TRUE, quote=FALSE, col.names=FALSE, row.names=FALSE, sep=" ") 

    # get meteo data
    fid <- nc_open(sDataFile)
      lonsite  <- ncvar_get(fid,"longitude")
      latsite  <- ncvar_get(fid,"latitude")
      dateLeap <- as.integer(ncvar_get(fid,"doy_leap"))    # get the dates as integer 0/1 information, where 1 indicates the 29th Feb in leap years
      
      swdown   <- ncvar_get(fid,"SWdown")
      lwdown   <- ncvar_get(fid,"LWdown")
      tair     <- ncvar_get(fid,"Tair")                # [K]
      rain     <- ncvar_get(fid,"Precip") * 86400      # precip = rain + snow
      qair     <- ncvar_get(fid,"Qair")   * 1000
      press    <- ncvar_get(fid,"Psurf")  / 100
      wind     <- ncvar_get(fid,"Wind")
      wind[wind < 0.1] <- 0.1    # may not be smaller 0.1 for model stability
      ## forcing input files do not contain snowfall information; QUINCY reads the snow, but does not use it; so, simply set all to zero
      snow    <- rain * 0.0 
    nc_close(fid)

    # remove leap days from meteo data (all Febr 29th)
    leapPositions <- grep("1", dateLeap)
    if(length(leapPositions) > 0){
      print("removing leap days from meteo data")
      swdown  <- swdown[- leapPositions]
      lwdown  <- lwdown[- leapPositions]
      tair    <- tair[- leapPositions]
      rain    <- rain[- leapPositions]
      qair    <- qair[- leapPositions]
      press   <- press[- leapPositions]
      wind    <- wind[- leapPositions]
      snow    <- snow[- leapPositions]
      # remove date vector; not needed hereafter
      remove(dateLeap)
    }

    # in case longitude needs to be converted back to the range [-180,180]
    if (lonsite > 180){ 
      lonsite <- lonsite - 360
      print(paste0("longitude converted to the range [-180,180], is now: ", lonsite))
    }

    # P deposition data
    fid <- nc_open("/Net/Groups/BGI/work_2/QUINCY/SLM/InputDataSources/P-DEP/nitrogenandphosphorus2x2annualdep.nc")
      lonpdep  <- as.numeric(ncvar_get(fid,'lon'))
      latpdep  <- as.numeric(ncvar_get(fid,'lat'))
      latx     <- which.min(abs(latpdep-latsite))
      lonx     <- which.min(abs(lonpdep-lonsite))
      # P deposition constant value
      p_dummy  <- ncvar_get(fid,'pdep')[lonx,latx]/365
      pdep     <- numeric(length(swdown)) 
      pdep[]   <- p_dummy
    nc_close(fid)

    # create empty vectors used for output data
    yTimeSteps <- 365*24*2              # 30 min time steps in QUINCY
    year_out   <- numeric(yTimeSteps)
    co2        <- numeric(yTimeSteps)
    co2_dC13   <- numeric(yTimeSteps)
    co2_DC14   <- numeric(yTimeSteps)
    noy        <- numeric(yTimeSteps)
    nhx        <- numeric(yTimeSteps)
    ycount     <- 1
    
    # loop over the years from default start of forcing (1901) to the site-specific last years (overall max should be 2014)
    for (year in forcing_startyear:endyear){

      ## N deposition
      #  this data covers the period 1850 to 2100
      if (year < 1996){
        # use Ndep data created by Sönke
        fid<-nc_open(paste0("/Net/Groups/BGI/model/OCN/input/gridded/NDEP/CESM-CAM/ndep_",year,".nc"))
      }
      else {
        #  using RCP 8.5 data for all years after 1995
        fid<-nc_open(paste0("/Net/Groups/BGI/model/OCN/input/gridded/NDEP/CESM-CAM/ndep_RCP8.5_",year,".nc"))
      }
        lonndep   <- ncvar_get(fid,'lon')
        latndep   <- ncvar_get(fid,'lat')
        latx      <- which.min(abs(latndep-as.numeric(latsite)))
        lonx      <- which.min(abs(lonndep-as.numeric(lonsite)))
        
        nhx_dummy <- ncvar_get(fid,'NHx_deposition')[lonx,latx,]*86400*1000*1000
        noy_dummy <- ncvar_get(fid,'NOy_deposition')[lonx,latx,]*86400*1000*1000
        nhx[1:(dpm[1]*48)] <- rep(nhx_dummy[1],dpm[1]*48)
        noy[1:(dpm[1]*48)] <- rep(noy_dummy[1],dpm[1]*48)
        
        for (m in 2:12) {
          nhx[(1:(dpm[m]*48))+sum(dpm[1:(m-1)])*48]<-rep(nhx_dummy[m],dpm[m]*48)
          noy[(1:(dpm[m]*48))+sum(dpm[1:(m-1)])*48]<-rep(noy_dummy[m],dpm[m]*48)
        }
      nc_close(fid)

      # CO2
      co2[1:yTimeSteps]      <- rep(co2y[co2y[,1]==year,2],yTimeSteps)
      co2_dC13[1:yTimeSteps] <- rep(co2y_dC13[(co2y_dC13[,1]-0.5)==year,2],yTimeSteps)
      if(latsite>30) C14lat=2
      if(latsite>-30&latsite<=30) C14lat=3
      if(latsite<=-30) C14lat=4
      co2_DC14[1:yTimeSteps] <- rep(co2y_DC14[(co2y_DC14[,1]-0.5)==year,C14lat],yTimeSteps)

      # year
      year_out[1:yTimeSteps] <- rep(year,yTimeSteps)

      # time information for output as defined above
      doy_out <- doy
      hod_out <- hod
      nstep   <- yTimeSteps
      
      # get actual year, differentiate between "transient period" and the period with actual fluxnet data
      if(year >= startyear & year <= endyear){
        climyear <- year-startyear+1            # current loop is: for (year in forcing_startyear:endyear)
      } else{
        climyear <- pickyear[ycount]
      }
   
      # combine individual variables into one array for output (note: nstep == yTimeSteps)
      forcing<-cbind(year_out[1:nstep],doy_out,hod_out,swdown[(1:nstep)+yTimeSteps*(climyear-1)],lwdown[(1:nstep)+yTimeSteps*(climyear-1)],
                     tair[(1:nstep)+yTimeSteps*(climyear-1)],qair[(1:nstep)+yTimeSteps*(climyear-1)],
                     press[(1:nstep)+yTimeSteps*(climyear-1)],rain[(1:nstep)+yTimeSteps*(climyear-1)],
                     snow[(1:nstep)+yTimeSteps*(climyear-1)],wind[(1:nstep)+yTimeSteps*(climyear-1)],co2[1:nstep],co2_dC13[1:nstep],co2_DC14[1:nstep],
                     nhx[1:nstep],noy[1:nstep],pdep[1:nstep])

      # write output (add another year of data)
      write.table(format(round(forcing,digits=4),nsmall=1), file=outname, append=TRUE, quote=FALSE,
                  col.names=FALSE, row.names=FALSE, sep=" ")

      # increase year counter
      ycount <- ycount+1

      # debug output a the screen
      print(paste0("Created data for site ",site_list[s]," and year ",year," using the fluxnet data from year ",climyear+startyear-1))
    }
  # if no data available for this site do not try to create met_forcing
  } else {
       print(paste0(sDataFile, ' met-forcing file for this site is not available'))
  }
} # loop over sites



## end ##
