##########################################################################################
##  this script generate a site list file of the Amazon site set for QUINCY simulations 
##  
##  Amazon sites are read from xxxx
##
##  run this script by:
##  cd contrib/forcing_scripts/
##  Rscript --vanilla ./produce_amazon_site_info_file.R
## 
##  script creates ./generate_amazon_site_list.dat  (columns: Site-ID lon lat PFT start end)
## 
##  output will be generated per default in:
##  ./new_amazon_transient_met_forcing/
##  output file contains "_t_" as identifier for transient forcing
## 
##  script was adapted for Amazon site set by Katrin Fleischer
##
#########################################################################################

## library
library(ncdf4)

#########################################################################################
## input path for original Amazon site set list
#ipath_info       <- "/Net/Groups/BGI/work_2/QUINCY/SLM/Data/amazon_site_set/"
ipath_info       <- "/Net/Groups/BGI/scratch/kfleischer/AMAZON/Amazon_siteset_lists/"
dat              <- read.table(paste0(ipath_info,"table_all_sites_quesada_siteinfo.csv"),sep=(","),stringsAsFactor=F,header=T)

# number of sites
siteNum <- nrow(dat)

## assign site names, lon/lat, PFT, start and end year
siteID_list     <- dat$site   
lon_orig        <- dat$lon
lat_orig        <- dat$lat
pft             <- rep("TrBE",siteNum)
start           <- rep(1901,siteNum)
end             <- rep(2019,siteNum)

plantyear       <- c(rep(-9999,siteNum))

## feedback
print(paste0(" Start creating the site_list info file for ",siteNum," sites of the Amazon siteset."))

## create new (empty) vectors/arrays for output
depth_begin  <- c(0.0,0.05,0.15,0.3,0.6,1.0,2.0)  
depth_end    <- c(0.05,0.15,0.3,0.6,1.0,2.0,100)
depth_sl     <- depth_end - depth_begin
awc_sl       <- array(NA,c(siteNum,7))
awc          <- numeric(siteNum)
depth        <- numeric(siteNum)
taxnwrb      <- numeric(siteNum)
taxousda     <- numeric(siteNum)

## assign soil characteristics
clay            <- dat$clay
silt            <- dat$silt
sand            <- dat$sand
ph              <- dat$pH
bulk_density    <- dat$bulk*1000 #original data is in g/m-3 and needed in mg/m-3
soilP_depth_site     <- rep(0.3, siteNum)
soilP_labile_site    <- rowSums(data.frame(dat$Plab,dat$Pbi_i,dat$Pbi_o))
soilP_slow_site      <- rowSums(data.frame(dat$Pna_i,dat$Pna_o))
soilP_occlud_site    <- dat$Pres
soilP_primary_site   <- dat$Pmin

# print out number of entries per variable that need supplementary from gridded products
print(paste0("Site data has been added from field observations-file."))
print(paste0("Soil texture values not available = ", sum(is.na(clay))))
print(paste0("Soil pH values not avialable = ", sum(is.na(ph))))
print(paste0("Soil bulk density values not available = ", sum(is.na(bulk_density))))
print(paste0("Soil P fractions values not available = ", sum(is.na(soilP_labile_site))))


########################################################################################
## fill data gaps from Soil Grids Database
## new input path
ipath <- "/Net/Groups/BGI/data/DataStructureMDI/DATA/grid/Global/0d10_static/soilgrids/v0_5_1/Data/"

# note: sand silt and pH are available from the above site-level file

ncid  <- nc_open(paste0(ipath,"SNDPPT.soilgrid.3600.1800.7.nc"))
lon_sg<-ncvar_get(ncid,'longitude')
lat_sg<-ncvar_get(ncid,'latitude')
#sand_sg<-ncvar_get(ncid,'SNDPPT')
nc_close(ncid)

#ncid<-nc_open(paste(ipath,'SLTPPT.soilgrid.3600.1800.7.nc',sep=''))
#silt_sg<-ncvar_get(ncid,'SLTPPT')
#nc_close(ncid)

ncid<-nc_open(paste(ipath,'BLDFIE.soilgrid.3600.1800.7.nc',sep=''))
bulk_density_sg<-ncvar_get(ncid,'BLDFIE')
nc_close(ncid)

# ncid<-nc_open(paste(ipath,'PHIHOX.soilgrid.3600.1800.7.nc',sep=''))
# ph_sg<-ncvar_get(ncid,'PHIHOX')
# nc_close(ncid)

ncid<-nc_open(paste(ipath,'AWCtS.soilgrid.3600.1800.7.nc',sep=''))
fc_vol_sg<-ncvar_get(ncid,'AWCtS')
nc_close(ncid)

ncid<-nc_open(paste(ipath,'WWP.soilgrid.3600.1800.7.nc',sep=''))
pwp_vol_sg<-ncvar_get(ncid,'WWP')
nc_close(ncid)

ncid<-nc_open(paste(ipath,'BDRICM.soilgrid.3600.1800.nc',sep=''))
depth_to_bedrock_sg<-ncvar_get(ncid,'BDRICM')
nc_close(ncid)

ncid<-nc_open(paste(ipath,'TAXOUSDA_10km.soilgrid.3600.1800.nc',sep=''))
taxousda_sg<-ncvar_get(ncid,'TAXOUSDA_10km')
nc_close(ncid)

ncid<-nc_open(paste(ipath,'TAXNWRB.soilgrid.3600.1800.nc',sep=''))
taxnwrb_sg<-ncvar_get(ncid,'TAXNWRB')
nc_close(ncid)


## add soil classification and supplementary data from gridded soil products
for (s in 1:siteNum) {
  lonidx          <- which.min(abs(lon_sg-lon_orig[s]))
  latidx          <- which.min(abs(lat_sg-lat_orig[s]))
  taxousda[s]      <- taxousda_sg[lonidx,latidx]
  taxnwrb[s]      <- taxnwrb_sg[lonidx,latidx]
  
  depth[s]        <- depth_to_bedrock_sg[lonidx,latidx]/100
  
  #add bulk density for missing values
  if(is.na(bulk_density[s])) bulk_density[s] <- mean(bulk_density_sg[lonidx,latidx,])

  #add available soil water (or calculate based on field data later)
  awc_sl[s,]<-(fc_vol_sg[lonidx,latidx,]-pwp_vol_sg[lonidx,latidx,])/100
  awc[s]<-weighted.mean(awc_sl[s,],depth_sl,na.rm=TRUE)*depth[s]*1000
  
  # if(ph[s]==6.5) ph[s]<-mean(ph_sg[lonidx,latidx,])/10
  # note:
  # sand silt pH, are read & used from the above site_list.dat file
  # if(is.na(sand[s])) sand[s]<-weighted.mean(sand_sg[lonidx,latidx,],depth_sl,na.rm=TRUE)/100
  # if(is.na(silt[s])) silt[s]<-weighted.mean(silt_sg[lonidx,latidx,],depth_sl,na.rm=TRUE)/100
}


#########################################################################################
## read lithology map
## new input file
ncid <- nc_open("/Net/Groups/BGI/data/DataStructureMDI/DATA/grid/Global/0d50_static/GLiM/v1_0/Data/GLim.720.360.nc")

lon_glim        <- ncvar_get(ncid,'lon')
lat_glim        <- ncvar_get(ncid,'lat')
glim_class      <- ncvar_get(ncid,'GLim')
glim_class_site <- numeric(siteNum)
nc_close(ncid)

## loop over sites: get soil data
for (s in 1:siteNum) {
  lonidx               <- which.min(abs(lon_glim-lon_orig[s]))
  latidx               <- which.min(abs(lat_glim-lat_orig[s]))
  glim_class_site[s]   <- glim_class[lonidx,latidx]
}

## replace NAN with default values

# sand[is.na(sand)] <- 0.4
# silt[is.na(silt)] <- 0.4
# clay              <- 1 - sand - silt
# ph[is.na(ph)]     <- 6
bulk_density[is.na(bulk_density)]                  <- 1500.0000001
awc[is.na(awc)]                <- 200.0000001
taxousda[is.na(taxousda)]      <- 30
taxnwrb[is.na(taxnwrb)]        <- 27
glim_class_site[is.na(glim_class_site)]    <- 2


#########################################################################################
## get site-specific data on qmax_org_fine_particle
## qmax_org_fine_particle was calculated by Lin (lyu@bgc-jena.mpg.de) per NWRB category available from TAXNWRB.soilgrid.3600.1800.nc
## two new input files
inNWRBcatFile     <- "/Net/Groups/BGI/data/DataStructureMDI/DATA/grid/Global/0d10_static/soilgrids/v0_5_1/Data/TAXNWRB.soilgrid.3600.1800.nc"
inCalcQmaxorgFile <- "qmax_org_values_per_nwrb_category_20180515.csv"


ncid         <- nc_open(inNWRBcatFile)  # grid with 3600x1800 resolution
lon_sg       <- ncvar_get(ncid,'longitude')
lat_sg       <- ncvar_get(ncid,'latitude')
nwrbCat_sg   <- ncvar_get(ncid,"TAXNWRB")
Qmax_org_fp  <- numeric(siteNum)
nc_close(ncid)

qmaxorgTable <- read.table(inCalcQmaxorgFile, header=T, sep="", stringsAsFactors=F)

## loop over sites: get NWRB cat data from nearest location & get according Qmax_org
for (s in 1:siteNum) {
  # get neares lon/lat in soil map
  lonidx <- which.min(abs(lon_sg-lon_orig[s]))
  latidx <- which.min(abs(lat_sg-lat_orig[s]))
  Qmax_org_fp[s] <- qmaxorgTable$qmax_org_value[ as.integer(nwrbCat_sg[lonidx,latidx]) ]
  # debug output
  #print(paste0("site: ",s," ",qmaxorg_site[s]))
}

# round to 4 decimal places
Qmax_org_fp <- round(Qmax_org_fp,4)

#########################################################################################
## start creating the final output file:
#  for transient 

# output file info & creation of empty object
outfileNamesBase  <- "generate_amazon_sites"
siteParamNum      <- 27
siteinfo          <- array(NA,c(siteNum, siteParamNum))
startYearTrans    <- 1901
lastYear          <- 2019

# feedback
print(" All data are gathered. Start creating output files.")

# loop over all ICP sites
for (i in 1:siteNum) {
  siteinfo[i,1]  <- siteID_list[i]
  siteinfo[i,2]  <- lon_orig[i]
  siteinfo[i,3]  <- lat_orig[i]
  siteinfo[i,4]  <- pft[i]
  siteinfo[i,5]  <- startYearTrans
  siteinfo[i,6]  <- lastYear
  siteinfo[i,7]  <- lon_orig[i]
  siteinfo[i,8]  <- clay[i]
  siteinfo[i,9]  <- silt[i]
  siteinfo[i,10] <- sand[i]
  siteinfo[i,11] <- awc[i]
  siteinfo[i,12] <- bulk_density[i]
  siteinfo[i,13] <- ph[i]
  siteinfo[i,14] <- taxousda[i]
  siteinfo[i,15] <- taxnwrb[i]
  siteinfo[i,16] <- glim_class_site[i]
  siteinfo[i,17] <- -9999   #LAI
  siteinfo[i,18] <- -9999   #Nleaf
  siteinfo[i,19] <- -9999   #SLA
  siteinfo[i,20] <- -9999   #Height
  siteinfo[i,21] <- plantyear[i]
  siteinfo[i,22] <- soilP_depth_site[i]
  siteinfo[i,23] <- soilP_labile_site[i]
  siteinfo[i,24] <- soilP_slow_site[i]
  siteinfo[i,25] <- soilP_occlud_site[i]
  siteinfo[i,26] <- soilP_primary_site[i]
  siteinfo[i,27] <- Qmax_org_fp[i]
}

## output
#  header (must match siteParamNum)
colnames(siteinfo)<-c('Site-ID','lon','lat','PFT','start','end',
                      'lon_gmt','clay','silt','sand','awc','bd','ph',
                      'taxousda','taxnwrb','lith_glim',
                      'LAI','Nleaf','SLA','Height','PlantYear',
                      'soilP_depth','soilP_labile','soilP_slow','soilP_occluded','soilP_primary',
                      'Qmax_org_fp')

# transient
outfileName <- paste0(outfileNamesBase,"_", startYearTrans, "-", lastYear, "_list")
print(paste0(" Writing the file ",outfileName,".dat/.csv"))
write.table(siteinfo, file=paste0(outfileName,".dat"), quote=F, row.names=F, col.names=T)
write.table(siteinfo ,file=paste0(outfileName,".csv"), quote=F, row.names=F, col.names=T) 

print(paste0("Script has finished. The new file with ",siteParamNum," parameters was created."))

## end ##

