#!/usr/bin/env Rscript

##    QUINCY terrestrial biosphere model with coupled carbon, nitrogen, and phosphorus cycles
##    Copyright (C) 2019 Max Planck Institute for Biogeochemistry (MPI-BGC), Jena, Germany 
##    Contributors: Caldararu S., Engel J., Kern M., Pallandt M., Schnur R., Thum T., Yu L., Zaehle S.
##
##    This file is part of the QUINCY model.
##    The QUINCY model is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License version 3,
##    as published by the Free Software Foundation.
##
##    The QUINCY model is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with the QUINCY model.  If not, see <https://www.gnu.org/licenses/>.

# Interface 
Args <- commandArgs(trailingOnly=TRUE);
if(length(Args) != 3) {
  message("slm2ncdf.R requires longitude latitude and startyear as input. Terminating");quit()
}
lon <- as.numeric(Args[1])
lat <- as.numeric(Args[2])
year <- as.numeric(Args[3])

library(ncdf4)

#function to creat cdf compliant files
create_cdf<-function(year,lon,lat,timeout,timeindex,fdata,miss_val,nvar,
                      fname,flongname,funit,fprec,fname_cdf) {

  #create dimensions
  dimLon<-ncdim_def("longitude","degrees_east",lon,unlim=FALSE,create_dimvar=TRUE)
  dimLat<-ncdim_def("latitude","degrees_north",lat,unlim=FALSE,create_dimvar=TRUE)
  dimTime<-ncdim_def("time",paste("seconds since ",year[timeindex][1],"-01-01 00:00:00",sep=""),
                      timeout,unlim=TRUE,create_dimvar=TRUE)

  #create variables
  forvarid<-list()
  for (v in 1:nvar) {
    forvarid[[v]]<-ncvar_def(fname[v],funit[v],list(dimLon,dimLat,dimTime),
                                missval=miss_val,longname=flongname[v],prec=fprec[v])
  }

  #create and write file
  ncid<-nc_create(fname_cdf,forvarid)
  for (v in 1:nvar) {
     ncvar_put(ncid,forvarid[[v]],fdata[v,timeindex])
     ncatt_put(ncid,forvarid[[v]],"associate","time latitude longitude",prec="text")
     ncatt_put(ncid,forvarid[[v]],"axis","TYX",prec="text")
  }
  ncatt_put(ncid,0,"description","SLM derived FLUXNET simulation",prec='text')
  ncatt_put(ncid,0,"conventions","CDF",prec='text')
  ncatt_put(ncid,0,"date",date(),prec='text')
  nc_close(ncid)

}


fluxCvegfile<-read.table('vegfluxC_timestep.txt',header=T)
fluxCsoilfile<-read.table('soilfluxCdC13DC14_timestep.txt',header=T)
fluxH2Ofile<-read.table('vegfluxH2O_timestep.txt',header=T)
fluxEfile<-read.table('vegfluxEnergy_timestep.txt',header=T)

miss_val <- -9.33e30
ntime<-length(fluxCvegfile[,1])
dt=round((fluxCvegfile[3,1]-fluxCvegfile[2,1])*86400)

variable_name <- array(c("PPFD","Rn","Qh","Qle","Ga","Gc","GPP","Reco","NEE","WUE","FAPAR"));nvar<-length(variable_name)
outname=outlongname=outunit=outprec=numeric(nvar)
outprec[]<-"double"
outvar<-array(0,c(nvar,ntime))
tstepaxis<-seq(1,ntime,1)
timeaxis<-tstepaxis*dt

fname<-'slm4fluxnet.nc'

for (v in 1:nvar) {
  outname[v]<-variable_name[v]
  if(variable_name[v]=='PPFD') {
     outlongname[v]<-"Photosynthetic Photon Flux Density"
     outunit[v]<-"µmol / m2 /s"
     outvar[v,]<-fluxEfile$PPFD
  }
  if(variable_name[v]=='Rn') {
     outlongname[v]<-"Net Radiation"
     outunit[v]<-"W / m2"
     outvar[v,]<-fluxEfile$Rnet
  }
  if(variable_name[v]=='Qh') {
     outlongname[v]<-"Sensible Heat FLux"
     outunit[v]<-"W / m2"
     outvar[v,]<-fluxEfile$Qh
  }
  if(variable_name[v]=='Qle') {
     outlongname[v]<-"Latent Heat FLux"
     outunit[v]<-"W / m2"
     outvar[v,]<-fluxEfile$Qle
  }
  if(variable_name[v]=='Ga') {
     outlongname[v]<-"Aerodynamic Conductance"
     outunit[v]<-"m/s"
     outvar[v,]<-fluxH2Ofile$GA
  }
  if(variable_name[v]=='Gc') {
     outlongname[v]<-"Canopy Conductance"
     outunit[v]<-"m/s"
     outvar[v,]<-fluxH2Ofile$GC
  }
  if(variable_name[v]=='GPP') {
     outlongname[v]<-"Gross Primary Production"
     outunit[v]<-"µmol C / m2 / s"
     outvar[v,]<-fluxCvegfile$GPP
  }
  if(variable_name[v]=='Reco') {
     outlongname[v]<-"Ecosystem Respiration"
     outunit[v]<-"µmol C / m2 / s"
     outvar[v,]<-fluxCvegfile$GrowthResp+fluxCvegfile$MaintResp+fluxCsoilfile$HetResp
  }
  if(variable_name[v]=='NEE') {
     outlongname[v]<-"Net Ecosystem Production"
     outunit[v]<-"µmol C / m2 / s"
     outvar[v,]<-fluxCsoilfile$HetResp-fluxCvegfile$NPP
  }
  if(variable_name[v]=='WUE') {
     outlongname[v]<-"Water-Use Efficiency"
     outunit[v]<-"gC / kg H20"
     outvar[v,]<-fluxCvegfile$GPP/(fluxH2Ofile$Transpiration+fluxH2Ofile$Interception+fluxH2Ofile$Evaporation)*12/1000000
     filt<-(fluxH2Ofile$Transpiration+fluxH2Ofile$Interception+fluxH2Ofile$Evaporation)
     filt[filt<0.00001]<-0
     filt[filt>=0.00001]<-fluxH2Ofile$Transpiration[filt>=0.00001]/filt[filt>=0.00001]
     outvar[v,filt<0.7]<-NA
     #outvar[v,is.na(outvar[v,])]<-0
  }
  if(variable_name[v]=='FAPAR') {
     outlongname[v]<-"Fraction of Absorbed Photosynthetically Active Radiation"
     outunit[v]<-"-"
     outvar[v,]<-fluxEfile$fAPPFD
     #outvar[v,is.na(outvar[v,])]<-0
  }

}

create_cdf(year,lon,lat,timeaxis,,outvar,miss_val,nvar,outname,outlongname,outunit,outprec,fname)
