#!/bin/bash

# documentation:
#   * this script does compare the output of a QUINCY simulation of ONE SITE against a reference run (develop branch)
#     * for the usecases 32/34/80/81/82 the latest reference run (develop branch) is used automatically
#     * if you want to compare your usecases 32/34/80/81/82 against a custom reference run, just set the argument 1 to "none"
#   * three arguments need to be provided to this script, plus two optional arguments: 
#     * the usecase, and the path to & directory of your simulation (see info below)
#     * the path & directory of the reference run is determined automatically in case your simulation was of the above usecases
#     * you can provide a custom reference run by simply use the (absolute) path to its main dir as argument 4 and set the output period as argument 5
#   * the code of this script does not need to be changed - the script-arguments provide sufficient information for the script to run
#   * no error checking is done for the script-arguments, except for the usecase number

# usage:
#   * two ways for using this script:
#   * run by: './compare_two_quincy_simulations.sh arg1 arg2 arg3 [arg4 arg5]'
#     * manually/interactively: execute it from a development node, e.g. io1, for one site of your simulation-experiment
#     * automatically: executed for each individual site from run_slm.sh when a site-level simulation-experiment is made with the usecases supported

# script arguments:
#   1 usecase number ( set reference run automatically for: 32 34 80 81 82 )
#   2 experiment directory of new simulation (with a trailing "/") [ e.g. /Net/Groups/BSI/scratch/"$USER"/quincy_exp/exp_82_baseline_fluxnet_stdBNF_P_68bfefb7/ ]
#   3 site ID (directory name for this simulation within the experiment dir) [ e.g. DE-Hai ]
#   4 optional: experiment directory of custom reference simulation (with a trailing "/"), i.e., if no usecase provided, such as, argument 1 is set to "none"
#   5 optional: output period [timestep daily weekly monthly yearly], needed (only) if the argument 4 is used
usecase="$1"
expMainDir="$2"
siteID="$3"
# $4 optional argument (tested below)
# $5 optional argument needed in addition to $4

## directories of reference runs
refRunMainDir="/Net/Groups/BGI/work_2/QUINCY/SLM/reference_runs/"
refRunUsecase3_Dir="fluxnet3_siteset_u32_34/"
refRunUsecase8_Dir="fluxnet2_siteset_u80_81_82/"

## output dir and output file names for this script
# postprocessing dir of the site that is tested
outDir="postprocessing/"
# comparison and diff plots
outFileExt=".pdf"
outCompBaseFileName="$siteID""_compare_to_ref_"
outDiffBaseFileName="$siteID""_diff_prop_sim_new_minus_ref_"

## get reference run directory of the site
# differ between usecases and catch usecases for which no reference runs are available
if [[ $usecase -eq 32 ]] || [[ $usecase -eq 34 ]]; then
  refRunDir="$refRunMainDir""$refRunUsecase3_Dir"
elif [[ $usecase -eq 80 ]] || [[ $usecase -eq 81 ]] || [[ $usecase -eq 82 ]]; then
  refRunDir="$refRunMainDir""$refRunUsecase8_Dir"
else
  # test for arg4 
  if [[ ! -z "$4" ]]; then     # if argument 4 is not empty
    # use directory of custom reference run
    refRunDir="$4"
    if [[ ! -d ${refRunDir} ]]; then
      echo " Unfortunately a not existing directory has been passed as forth argument."
      echo " Please check the passed directory: ""$refRunDir""."
      echo " EXIT!"
      exit 1
    fi
    # set output period
    outputPeriodCustom="$5"
    if [[ ${outputPeriodCustom} == "" ]]; then
      echo " If a custom reference run is specified, the output period of the runs needs to be specified."
      echo " Please specify the output period of the to be compared runs as fifth argument."
      echo " EXIT!"
      exit 1
    fi
    # for custom reference run, set usecase variable to specific key word expected below
    usecase="none"  
  # if a unsupported usecase is provided as arg1
  else
    echo " For the usecase passed as argument no reference run is available."
    echo " Passed usecase: '""$usecase""'. Supported are: 32/34 & 80/81/82."
    echo " EXIT!"
    exit 1
  fi
fi


## reference run with usecases
if [[ $usecase != "none" ]]; then
  # get latest ref run directory - assuming the directories always start with the date (YYYYMMDD_...)
  refRunDir="$(ls -d -1 "$refRunDir""20"* | tail -n 1)""/"
  # add experiment dir
  refRunDir="$(ls -d -1 "$refRunDir""exp_""$usecase""_"*)""/"
fi

## reference run with usecase and custom
# extract git commit number - reference run
refRunCommitNumber="$(echo $refRunDir | rev | cut -c 1-9 | rev | cut -c 1-8)"
# add the siteID
refRunSiteDir="$refRunDir""$siteID""/"

## new simulation run
# extract git commit number
newSimCommitNumber="$(echo $expMainDir | rev | cut -c 1-9 | rev | cut -c 1-8)"
# get path/directory/ of the site
newSimRunSiteDir="$expMainDir""$siteID""/"

## feedback
echo " comparing the simulation of one site with the recent reference run:"
echo "  usecase: ""$usecase"
echo "  ref: ""$refRunSiteDir"
echo "  sim: ""$newSimRunSiteDir"


## ------------------------------------------------------------------------------------------------
## run the R script to compare data
R --slave --silent --no-save << EOF

setwd("$newSimRunSiteDir")

# info text about git versions
info_text <- paste0("Compare new sim from ", "$newSimCommitNumber", " against ref from ", "$refRunCommitNumber", " for ", "$siteID", " ")

# quincy model output 
if ("$usecase" != "none"){
  outputPeriod <- c("monthly", "timestep")  
} else{
  outputPeriod <- "$outputPeriodCustom"
}
outputFileType <- ".txt"
simOutputFiles <- c("vegpoolC", "vegpooldC13DC14", "vegpoolN", "vegpooldN15", "vegpoolP", 
                    "soilpoolInOrg", "soilpoolOrg", "soilpooldC13DC14dN15", "soilpoolPhysics", 
                    "veg_diagnostics", 
                    "ecopoolCNP", 
                    "vegfluxC", "vegfluxdC13DC14", "vegfluxN", "vegfluxP", "vegfluxEnergy", "vegfluxH2O", 
                    "soilfluxCdC13DC14", "soilfluxN", "soilfluxdN", "soilfluxP", 
                    "sb_massBalance", 
                    "atm_forcing")
numberSimOutputFiles <- length(simOutputFiles)

# loop over output period
for (op in 1:length(outputPeriod)){

###########################################################
## plot data of old and new
###########################################################

# script output file name
outFileName <- paste0("$outCompBaseFileName", "$refRunCommitNumber", "_", outputPeriod[op], "$outFileExt")

print(" ")
print(paste0(">>>> creating comparison plots | ", outputPeriod[op]))
pdf(paste0("$newSimRunSiteDir", "$outDir", outFileName), paper="a4", height=12.4, width=7.6, pointsize=12, 
    bg="white", family="Helvetica", version="1.6")
  for (i in 1:numberSimOutputFiles)
  {
    # read files and print filename
    quincyOutFileNew <- paste0("output/", simOutputFiles[i], "_", outputPeriod[op], outputFileType)
    print(quincyOutFileNew)
    dat_new <- read.table(quincyOutFileNew, header=T, sep="")
    quincyOutFileOld <- paste0("$refRunSiteDir", "output/", simOutputFiles[i], "_", outputPeriod[op], outputFileType)
    print(quincyOutFileOld)
    dat_old <- read.table(quincyOutFileOld, header=T, sep="")
    # number of columns
    colNumN <- ncol(dat_new)
    colNumO <- ncol(dat_old)

    # file name without directory and file extension
    fNameN  <- simOutputFiles[i]
    
    # check whether column-names match (i.e., names & sequence of variables)
    # skip this file if column names of 'old' and 'new' are no match (using the R statement 'next')
    if (colNumN != colNumO)
    {
      print(paste0("   Number of variables differ between reference and new output file: ", fNameN, " - skipping this file."))
      next
    }
    if ( all(colnames(dat_new) != colnames(dat_old)) )
    {
      print(paste0("   Names of variables differ between reference and new output file: ", fNameN, " - skipping this file."))
      next
    }
    
    # collect information to distribute figs over multiple pages if more than 14
    # the number of variables is the 'number of columns - 1', as the 1st column is always 'time'
    varNum   <- colNumN - 1
    maxFigPg <- 14
    if (varNum < (maxFigPg + 1))
    {
      # page numbers
      pagesNum <- 1
      # column numbers of output file per page 
      colNumStarts <- 2
      colNumEnds   <- colNumN
    } else{
      # page numbers
      pagesNum <- as.integer(varNum / maxFigPg)  # number of pages minus 1 | == pages with 14 figs each
                                                 # last page may has less figs
      # test if there is a last page with /= 14 figs, and increase pagesNum if true 
      if ((varNum / maxFigPg) > pagesNum)
      {
        pagesNum <- pagesNum + 1
      }
      # column numbers of output file per page 
      colNumLast   <- colNumN
      colNumStarts <- seq(2, colNumLast, maxFigPg)    # e.g., seq(2,37,14) = 2 16 30
      colNumEnds   <- colNumStarts[2:length(colNumStarts)] - 1
      colNumEnds   <- c(colNumEnds, colNumLast)      
    }

    # create figures per page
    for (p in 1:pagesNum)
    {
      # get the effective (actual) number of variables at this page
      effVarNum <- colNumEnds[p] - colNumStarts[p] + 1

      # page layout
      # see also layout information in below 'plot()' command
      # set the minimum of effVarNum to 4 (so that figures get not too big if their number is)
      mat  <- matrix(seq(1,2*round(max(effVarNum,4)/2)+2,1), nrow=round(max(effVarNum,4)/2)+1, ncol=2, byrow=T) 
      layout(mat)
      par(mai=c(0.32, 0.4, 0.14, 0.02), oma=c(0.1, 0.1, 2.0, 0.1)) # oma = outer margins  || c(bottom, left, top, right)

      # loop over columns (variables) and create figs   
      idInfo1 <- "black -> reference run"
      idInfo2 <- "orange -> new simulation"   
      for (v in colNumStarts[p]:colNumEnds[p])
      {
        # plot simulation ID info at uppermost figures per page
        idInfo <- ""
        if(v == colNumStarts[p])     idInfo <- idInfo1
        if(v == colNumStarts[p] + 1) idInfo <- idInfo2
        # set y-axis range to default values
        yMin <- -1
        yMax <-  1
        # test if all values are NA
        if(all(is.na(dat_new[,v])) | all(is.na(dat_old[,v]))){
          print(paste0("  creating a green zero-line diff-figure for ", colnames(dat_new)[v], " because all values of either old or new are NA"))
          plot(1:nrow(dat_new), rep(0, nrow(dat_new)),
               type='l',lwd=0.4, col="forestgreen", ylim=c(yMin,yMax), ylab=colnames(dat_new)[v], xlab=outputPeriod[op], main=idInfo, 
               tck=0.03, mgp=c(1.5, 0.2, 0))
          if(v == colNumStarts[p]) mtext(fNameN, side=3, srt = 90, cex=1.0, line=1.8, col="red3")
        } else if(all(is.na(dat_new[,v])) & all(is.na(dat_old[,v]))){
          print(paste0("  creating a green zero-line diff-figure for ", colnames(dat_new)[v], " because all values of old and new are NA"))
          plot(1:nrow(dat_new), rep(0, nrow(dat_new)),
               type='l',lwd=0.4, col="forestgreen", ylim=c(yMin,yMax), ylab=colnames(dat_new)[v], xlab=outputPeriod[op], main=idInfo, 
               tck=0.03, mgp=c(1.5, 0.2, 0))
          if(v == colNumStarts[p]) mtext(fNameN, side=3, srt = 90, cex=1.0, line=1.8, col="red3")
        } else{
          # y-axis min & max
          yMin <- min( c(min(dat_new[,v], na.rm=T), min(dat_old[,v], na.rm=T)) )
          yMax <- max( c(max(dat_new[,v], na.rm=T), max(dat_old[,v], na.rm=T)) )
          # plot
          plot(dat_old[,v], 
               type='l',lwd=0.4, col="black", ylim=c(yMin,yMax), ylab=colnames(dat_old)[v], xlab=outputPeriod[op], main=idInfo, 
               tck=0.03, mgp=c(1.5, 0.2, 0))
          if(v == colNumStarts[p]) mtext(fNameN, side=3, srt = 90, cex=1.0, line=1.8, col="red3")
          lines(dat_new[,v], lwd=0.4, col="orange")
        }
        mtext(info_text, outer=TRUE,  cex=0.75, line=-0.75, side=1, adj=1, font=1)
      }
    } # p in 1:pagesNum
  }   # for i in 1:nFiles
garbage <- dev.off()  # run dev.off() but suppress the notification it usually provides

###########################################################
## plot proportional diff-data 'old - new'
###########################################################

# script output file name
outFileName <- paste0("$outDiffBaseFileName", outputPeriod[op], "$outFileExt")

print(" ")
print(paste0(">>>> creating diff plots | ", outputPeriod[op]))
pdf(paste0("$newSimRunSiteDir", "$outDir", outFileName), paper="a4", height=12.4, width=7.6, pointsize=12, 
    bg="white", family="Helvetica", version="1.6")
  for (i in 1:numberSimOutputFiles)
  {
    # read files and print filename
    quincyOutFileNew <- paste0("output/", simOutputFiles[i], "_", outputPeriod[op], outputFileType)
    print(quincyOutFileNew)
    dat_new <- read.table(quincyOutFileNew, header=T, sep="")
    quincyOutFileOld <- paste0("$refRunSiteDir", "output/", simOutputFiles[i], "_", outputPeriod[op], outputFileType)
    print(quincyOutFileOld)
    dat_old <- read.table(quincyOutFileOld, header=T, sep="")
    # number of columns
    colNumN <- ncol(dat_new)
    colNumO <- ncol(dat_old)

    # file name without directory and file extension
    fNameN  <- simOutputFiles[i]

    # check whether column-names match (i.e., names & sequence of variables)
    # skip this file if column names of 'old' and 'new' are no match (using the R statement 'next')
    if (colNumN != colNumO)
    {
      print(paste0("   Number of variables differ between old and new output file: ", fNameN, " - skipping this file."))
      next
    }
    if ( all(colnames(dat_new) != colnames(dat_old)) )
    {
      print(paste0("   Names of variables differ between old and new output file: ", fNameN, " - skipping this file."))
      next
    }

    # calc proportional difference
    # show the proportional difference of the new to the old simulation, based on the old simulation data
    # calc absolute diff
    dat_diff <- dat_new - dat_old
    # set all 0 in dat_old == NA to avoid division by zero
    # info: in plot() the NA will be simply left out
    dat_old[dat_old == 0] <- NA
    # calc proportional diff
    dat_diff <- (dat_diff / dat_old) * 100

    # collect information to distribute figs over multiple pages if more than 14
    # the number of variables is the 'number of columns - 1', as the 1st column is always 'time'
    varNum   <- colNumN - 1
    maxFigPg <- 14
    if (varNum < (maxFigPg + 1))
    {
      # page numbers
      pagesNum <- 1
      # column numbers of output file per page 
      colNumStarts <- 2
      colNumEnds   <- colNumN
    } else{
      # page numbers
      pagesNum <- as.integer(varNum / maxFigPg)  # number of pages minus 1 | == pages with 14 figs each
                                                 # last page may has less figs
      # test if there is a last page with /= 14 figs, and increase pagesNum if true 
      if ((varNum / maxFigPg) > pagesNum)
      {
        pagesNum <- pagesNum + 1
      }
      # column numbers of output file per page 
      colNumLast   <- colNumN
      colNumStarts <- seq(2, colNumLast, maxFigPg)    # e.g., seq(2,37,14) = 2 16 30
      colNumEnds   <- colNumStarts[2:length(colNumStarts)] - 1
      colNumEnds   <- c(colNumEnds, colNumLast)      
    }

    # create figures per page
    for (p in 1:pagesNum)
    {
      # get the effective (actual) number of variables at this page
      effVarNum <- colNumEnds[p] - colNumStarts[p] + 1

      # page layout
      # see also layout information in below 'plot()' command
      # set the minimum of effVarNum to 4 (so that figures get not too big if their number is)
      mat  <- matrix(seq(1,2*round(max(effVarNum,4)/2)+2,1), nrow=round(max(effVarNum,4)/2)+1, ncol=2, byrow=T) 
      layout(mat)
      par(mai=c(0.32, 0.4, 0.14, 0.02), oma=c(0.1, 0.1, 2.0, 0.1)) # oma = outer margins  || c(bottom, left, top, right)

      # loop over columns (variables) and create figs   
      idInfo1 <- "diff: (new - old) / old * 100"
      idInfo2 <- "ref run | new sim"
      for (v in colNumStarts[p]:colNumEnds[p])
      {
        # plot simulation ID info at uppermost figures per page
        idInfo <- ""
        if(v == colNumStarts[p])     idInfo <- idInfo1
        if(v == colNumStarts[p] + 1) idInfo <- idInfo2
        # set y-axis range to default values
        yMin <- -1
        yMax <-  1
        # test if all values are NA
        if(all(is.na(dat_diff[,v])))
        {
          print(paste0("  creating a green zero-line diff-figure for ", colnames(dat_diff)[v], " because all values are NA"))
          plot(1:nrow(dat_diff), rep(0, nrow(dat_diff)), 
               type='l',lwd=0.4, col="forestgreen", ylim=c(yMin,yMax), ylab=colnames(dat_diff)[v], xlab=outputPeriod[op], main=idInfo, 
               tck=0.03, mgp=c(1.5, 0.2, 0))
          if(v == colNumStarts[p]) mtext(fNameN, side=3, srt = 90, cex=1.0, line=1.8, col="red3")
        } else{
          # get useful min/max of y-axis
          yMin <- min(dat_diff[,v], na.rm = T)
          yMax <- max(dat_diff[,v], na.rm = T)
          if(!is.na(yMin)){ 
            if(yMin < -120.0) yMin <- -120.0
          } else yMin <- -1
          if(!is.na(yMax)){ 
            if(yMax > 120.0) yMax <-  120.0
          } else yMax <- 1
          # plot
          plot(dat_diff[,v], 
               type='l',lwd=0.4, col="black", ylim=c(yMin,yMax), ylab=colnames(dat_diff)[v], xlab=outputPeriod[op], main=idInfo, 
               tck=0.03, mgp=c(1.5, 0.2, 0))
          if(v == colNumStarts[p]) mtext(fNameN, side=3, srt = 90, cex=1.0, line=1.8, col="red3")
          abline(h=c(-4,4), lwd=0.8, col="red")
        }
        mtext(info_text, outer=TRUE,  cex=0.75, line=-0.75, side=1, adj=1, font=1)
      }
    } # p in 1:pagesNum
  }   # for i in 1:nFiles
garbage <- dev.off()  # run dev.off() but suppress the notification it usually provides

} # end loop over output period

# message
print(" ")
print(" finished creating comparison and diff figures!")

EOF
## end of R script

## fin
