!>
!! @par Copyright
!! This code is subject to the MPI-M-Software - License - Agreement in it's most recent form.
!! Please see URL http://www.mpimet.mpg.de/en/science/models/license .
!!
!! @brief Contains concrete tile class that contains the surface structure and memory state.
!!
!! @author
!!  Reiner Schnur, MPI-M Hamburg
!!
!! @par Revision History
!! First version by Reiner Schnur (2013-04-04)
!!
!! This had to be separated from mo_jsb_tile_class in order to avoid circular dependencies.
!!
MODULE mo_jsb_tile
#ifndef __NO_JSBACH__

  USE mo_kind,                ONLY: wp
  USE mo_exception,           ONLY: message_text, message, finish
  USE mo_jsb_control,         ONLY: debug_on
  USE mo_hsm_class,           ONLY: t_Hsm, t_State, t_Message
  USE mo_jsb_model_class,     ONLY: t_jsb_model
  USE mo_jsb_tile_class,      ONLY: t_jsb_tile_abstract, t_jsb_aggregator_p, t_jsb_aggregator
  USE mo_jsb_process_class,   ONLY: t_jsb_process_p, &
                                    SKIP_, AGGREGATE_, ON_LEAFS_, ON_TILE_, ON_SUBTREE_, INHERIT_
  USE mo_jsb_process_factory, ONLY: max_no_of_processes

  IMPLICIT NONE
  PRIVATE

  PUBLIC :: t_jsb_tile, t_jsb_tile_p

  TYPE, EXTENDS(t_jsb_tile_abstract) :: t_jsb_tile
    LOGICAL :: must_create_memory_parents(max_no_of_processes)    !< helper variable
    TYPE(t_jsb_model),         POINTER :: model => NULL()
  CONTAINS
    PROCEDURE :: Init      => Init_tile
  END TYPE t_jsb_tile

  TYPE t_jsb_tile_p
    TYPE(t_jsb_tile), POINTER :: p
  END TYPE t_jsb_tile_p

  ! Derived-type constructor
  INTERFACE t_jsb_tile
    PROCEDURE Construct_tile
  END INTERFACE t_jsb_tile

  CHARACTER(len=*), PARAMETER :: modname = 'mo_jsb_tile'

CONTAINS

  FUNCTION Construct_tile(name, description, processes, lct, &
    & process_actions, parent, model_id, grid_id, fract_filename, fract_varname) RESULT(return_ptr)

    USE mo_jsb_class,           ONLY: Get_model
    USE mo_util,                ONLY: one_of
    USE mo_jsb_lct_class,       ONLY: t_jsb_lct, LAND_TYPE, LAKE_TYPE, Contains_lct
    USE mo_jsb_process_class,   ONLY: SEB_, A2L_, L2A_, Get_process_name

    CHARACTER(len=*),  INTENT(in)           :: name
    CHARACTER(len=*),  INTENT(in)           :: description
    INTEGER,           INTENT(in), OPTIONAL :: processes(:)
    TYPE(t_jsb_lct),   INTENT(in), OPTIONAL :: lct
    INTEGER,           INTENT(in), OPTIONAL :: process_actions(:)
    CLASS(t_jsb_tile), POINTER,    OPTIONAL :: parent
    INTEGER,           INTENT(in), OPTIONAL :: model_id       !< Only needed if parent is not present
    INTEGER,           INTENT(in), OPTIONAL :: grid_id        !< Only needed if parent is not present
    CHARACTER(len=*),  INTENT(in), OPTIONAL :: fract_filename !< Name of file with tile fractions
    CHARACTER(len=*),  INTENT(in), OPTIONAL :: fract_varname  !< Name of fraction variable in fract_filename
    CLASS(t_jsb_tile), POINTER              :: return_ptr

    TYPE(t_jsb_model),            POINTER :: model
    CHARACTER(LEN=15)                     :: process_name
    INTEGER                               :: i, j, iproc
    CLASS(t_jsb_tile_abstract),   POINTER :: tmp_tile
    CLASS(t_State),               POINTER :: parent_tmp

    CHARACTER(len=*), PARAMETER :: routine = TRIM(modname)//':Construct_tile'

    IF (debug_on()) CALL message(TRIM(routine), 'Starting construction of tile '//TRIM(name))

    ! Allocate and initialize base structure (=state) for tile
    ALLOCATE(t_jsb_tile::return_ptr)
    IF (PRESENT(parent)) THEN
      parent_tmp => parent
      CALL return_ptr%Init_state(name, description, parent_tmp)
    ELSE
      CALL return_ptr%Init_state(name, description)
    END IF

    ! Initialize further components of t_jsb_tile_abstract extended type

    ALLOCATE(return_ptr%process_action(max_no_of_processes))

    IF (PRESENT(processes) .AND. PRESENT(process_actions)) THEN
      IF (SIZE(processes) /= SIZE(process_actions)) &
        & CALL finish(TRIM(routine), 'Size of processes and process_actions must be equal!')
    END IF

    IF (PRESENT(parent)) THEN
      SELECT TYPE (parent)
      CLASS IS (t_jsb_tile)
        return_ptr%parent_tile => parent
        return_ptr%grid_id = parent%grid_id
        return_ptr%owner_model_id = parent%owner_model_id
      END SELECT
    ELSE
      IF (PRESENT(model_id)) THEN
        return_ptr%owner_model_id = model_id
      ELSE
        CALL finish(TRIM(routine), 'Need model_id')
      END IF
      IF (PRESENT(grid_id)) THEN
        return_ptr%grid_id = grid_id
      ELSE
        CALL finish(TRIM(routine), 'Need grid_id')
      END IF
    END IF

    model => Get_model(return_ptr%owner_model_id)
    return_ptr%model => model

    IF (ASSOCIATED(return_ptr%parent_tile)) THEN
      IF (.NOT. ASSOCIATED(return_ptr%parent_tile%first_child_tile)) THEN
          return_ptr%parent_tile%first_child_tile => return_ptr
      END IF
    END IF
    IF (ASSOCIATED(return_ptr%prev_sibling)) THEN
      SELECT TYPE (prev_sibling => return_ptr%prev_sibling)
      CLASS IS (t_jsb_tile)
        prev_sibling%next_sibling_tile => return_ptr
      END SELECT
    END IF

    IF (PRESENT(lct)) THEN
      IF (ASSOCIATED(return_ptr%parent_tile)) THEN
        IF (ASSOCIATED(return_ptr%parent_tile%parent_tile) .AND. (lct%id == LAND_TYPE .OR. lct%id == LAKE_TYPE)) &
          & CALL finish(TRIM(routine), 'Tiles with lct LAND_TYPE or LAKE_TYPE must be direct children of root tile.')
      END IF
    END IF

      return_ptr%fract_filename = TRIM(fract_filename)

    IF (PRESENT(fract_varname)) THEN
      return_ptr%fract_varname = TRIM(fract_varname)
    ELSE
      return_ptr%fract_varname = ''
    END IF

    ! Loop over all processes and set actions for this tile
    return_ptr%must_create_memory_parents(:) = .FALSE.
    DO i=1,max_no_of_processes
      return_ptr%process_action(i) = SKIP_
      iproc = -1
      j = 0
      IF (PRESENT(processes)) THEN
        IF (SIZE(processes) > 0) THEN
          j = one_of(i, processes) ! i counts through the number of all processes, j is the number of the i-process
                                   ! within the processes vector (input)
        END IF
      END IF
      IF (j > 0) THEN
        iproc = processes(j)

        IF (PRESENT(process_actions)) THEN
          return_ptr%process_action(iproc) = process_actions(j)
        ELSE
          IF (iproc == A2L_) THEN
            return_ptr%process_action(iproc) = ON_TILE_
          ELSE IF (iproc == L2A_) THEN
            return_ptr%process_action(iproc) = ON_TILE_
          ELSE
            return_ptr%process_action(iproc) = ON_LEAFS_
          END IF
        END IF
        ! All parents of this tile aggregate memory for this new process
        return_ptr%must_create_memory_parents(i) = .TRUE.
!!$      ELSE IF (ASSOCIATED(return_ptr%prev_sibling)) THEN
!!$        iproc = i
!!$        SELECT TYPE (sibling => return_ptr%prev_sibling)
!!$        CLASS IS (t_jsb_tile_abstract)
!!$          return_ptr%process_action(iproc) = sibling%process_action(iproc)
!!$        END SELECT
      ELSE IF (ASSOCIATED(return_ptr%parent_tile)) THEN
        !IF (return_ptr%parent_tile%process_action(i) > AGGREGATE_) THEN
          iproc = i
          SELECT CASE (return_ptr%parent_tile%process_action(iproc))
          CASE (SKIP_)
            return_ptr%process_action(iproc) = SKIP_
          CASE (INHERIT_)
            return_ptr%process_action(iproc) = INHERIT_
          CASE (ON_SUBTREE_)
            return_ptr%process_action(iproc) = ON_SUBTREE_
          CASE (ON_LEAFS_)
            return_ptr%process_action(iproc) = ON_LEAFS_
            !return_ptr%parent_tile%process_action(iproc) = AGGREGATE_
          CASE (ON_TILE_)
            IF (iproc == L2A_) THEN
              return_ptr%process_action(iproc) = SKIP_
            ELSE
              return_ptr%process_action(iproc) = INHERIT_
            END IF
          END SELECT
        !END IF
      END IF

    END DO

    IF (debug_on()) CALL message(TRIM(routine), 'Finished construction of tile '//TRIM(name))

  END FUNCTION Construct_tile

  ! Note: It is important that Init_tile is only called for the tiles after the hierarchical tile tree has
  !       been completely constructed. Therefore, Init_tile is called from init_model_common.
  !
  SUBROUTINE Init_tile(this, varlist_name, prefix, suffix, l_fixed_fractions)

    USE mo_jsb_process_factory, ONLY: Create_process_memory
    USE mo_jsb_process_class,   ONLY: Get_process_name
    USE mo_jsb_memory_class,    ONLY: t_jsb_memory
    USE mo_jsb_tile_class,      ONLY: t_jsb_aggregator, t_jsb_aggregator_weighted_by_fract
    USE mo_jsb_varlist_iface,   ONLY: VARNAME_LEN
    USE mo_jsb_io,              ONLY: grib_bits, t_cf, t_grib1, t_grib2, tables ! , TSTEP_INSTANT, TSTEP_CONSTANT

    CLASS(t_jsb_tile), INTENT(inout) :: this
    CHARACTER(len=*),  INTENT(in)    :: varlist_name
    CHARACTER(len=*),  INTENT(in)    :: prefix         ! Prefix for variable names
    CHARACTER(len=*),  INTENT(in)    :: suffix
    LOGICAL,           INTENT(in)    :: l_fixed_fractions  ! Tile fractions can change or not

    INTEGER                               :: iproc
    CLASS(t_jsb_memory), POINTER          :: tmp_ptr
    CLASS(t_jsb_tile_abstract),   POINTER :: tmp_tile
    CLASS(t_State), POINTER               :: tmp_child
    CHARACTER(LEN=15)                     :: process_name

    TYPE(t_jsb_aggregator_weighted_by_fract), POINTER :: aggregator_weighted_by_fract
    CLASS(t_jsb_aggregator),                  POINTER :: aggregator
    REAL(wp),          POINTER :: return_pointer(:,:)
    CHARACTER(len=50)          :: name_loc
    CHARACTER(len=VARNAME_LEN) :: name
    INTEGER                    :: ilct, i, no_of_children, child_no

    INTEGER :: isteptype

    CHARACTER(len=*), PARAMETER :: routine = TRIM(modname)//':Init_tile'

    IF (debug_on()) CALL message(TRIM(routine), 'Initializing tile '//TRIM(this%name))

    ! Allocate all memory components for new tile and initialize their data part with NULL()
    ALLOCATE(this%mem(max_no_of_processes))

    ! Loop over all processes and initialize memory for this tile
    DO iproc=1,max_no_of_processes

      this%mem(iproc)%p => NULL()

      IF (this%process_action(iproc) == SKIP_) CYCLE

      IF (this%process_action(iproc) == ON_LEAFS_) THEN
        IF (ASSOCIATED(this%parent_tile)) this%parent_tile%process_action(iproc) = AGGREGATE_
        IF (.NOT. this%Has_children()) this%process_action(iproc) = ON_TILE_
      END IF

      process_name = Get_process_name(iproc)

      IF (this%process_action(iproc) == INHERIT_) THEN
        this%mem(iproc)%p => this%parent_tile%mem(iproc)%p
        IF (debug_on()) &
          & CALL message(TRIM(routine), 'Linked memory for process '//TRIM(process_name)//&
          &                             ' on tile '//TRIM(this%name)//' to parent')
      ELSE
        no_of_children = this%Get_no_of_children()
        child_no = this%path(this%level)
        ! this%param(iproc)%p => t_jsb_param(iproc)
        tmp_ptr => Create_process_memory(iproc)
        IF (ASSOCIATED(this%parent_tile)) THEN
          IF (.NOT. ASSOCIATED(this%parent_tile%mem(iproc)%p%children) .AND. this%parent_tile%Get_no_of_children() > 0) &
            & ALLOCATE(this%parent_tile%mem(iproc)%p%children(this%parent_tile%Get_no_of_children()))
          this%parent_tile%mem(iproc)%p%children(child_no)%p => tmp_ptr
        END IF
        this%mem(iproc)%p => tmp_ptr
        this%mem(iproc)%p%id = iproc
        this%mem(iproc)%p%grid_id = this%grid_id
        this%mem(iproc)%p%owner_model_id = this%owner_model_id
        this%mem(iproc)%p%owner_proc_id = iproc
        this%mem(iproc)%p%no_of_children = no_of_children
        IF (prefix /= '') THEN
          CALL this%mem(iproc)%p%Init(TRIM(process_name)//'_'//TRIM(prefix), TRIM(suffix), this%lcts%id, this%owner_model_id)
        ELSE
          CALL this%mem(iproc)%p%Init(TRIM(process_name), TRIM(suffix), this%lcts%id, this%owner_model_id)
        END IF
        IF (debug_on()) &
          & CALL message(TRIM(routine), 'Created memory for process '//TRIM(process_name)//' on tile '//TRIM(this%name))
      END IF

    END DO

    ! If tile has children, create and register aggregator
!    IF (this%Has_children()) THEN
      aggregator_weighted_by_fract => t_jsb_aggregator_weighted_by_fract(this)
      CALL this%Register_aggregator(aggregator_weighted_by_fract)
!    END IF

  END SUBROUTINE Init_tile

#endif
END MODULE mo_jsb_tile
