!>
!!  @par Copyright
!!    QUINCY terrestrial biosphere model with coupled carbon, nitrogen, and phosphorus cycles
!!    Copyright (C) 2019 Max Planck Institute for Biogeochemistry (MPI-BGC), Jena, Germany 
!!    Contributors: Caldararu S., Engel J., Kern M., Pallandt M., Schnur R., Thum T., Yu L., Zaehle S.
!!
!!    This file is part of the QUINCY model.
!!    The QUINCY model is free software: you can redistribute it and/or modify
!!    it under the terms of the GNU General Public License version 3,
!!    as published by the Free Software Foundation.
!!
!!    The QUINCY model is distributed in the hope that it will be useful,
!!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!    GNU General Public License for more details.
!!
!!    You should have received a copy of the GNU General Public License
!!    along with the QUINCY model.  If not, see <https://www.gnu.org/licenses/>.
!! 
!! @brief Contains structures and methods for sse config
!!
!! @author
!!  Reiner Schnur, MPI-M Hamburg
!!
!! @par Revision History
!! First version by Reiner Schnur (2013-04-04);
!! 2nd version by Jan Engel, MPI BGC (20160913) -> integration into SLM
!!
MODULE mo_sse_config_class
#ifndef __NO_JSBACH__

  USE mo_exception,         ONLY: message_text, message, finish
  USE mo_io_units,          ONLY: filename_max
  USE mo_kind,              ONLY: wp
  USE mo_util,              ONLY: real2string
  USE mo_jsb_config_class,  ONLY: t_jsb_config

  IMPLICIT NONE
  PRIVATE
  PUBLIC :: t_sse_config, max_soil_layers

  !-----------------------------------------------------------------------------------------------------
  !> configuration of the sse process, derived from t_jsb_config
  !! 
  !! currently it does mainly: reading parameters from namelist
  !-----------------------------------------------------------------------------------------------------
  TYPE, EXTENDS(t_jsb_config) :: t_sse_config
    INTEGER               :: nsoil_energy         !< number of soil layers for soil energy calculations   ! depreciated feature, but needed for now, see ticket #521
    INTEGER               :: nsoil_water          !< number of soil layers for soil moisture calculations ! depreciated feature, but needed for now, see ticket #521
    REAL(wp)              :: soil_depth           !< actual soil and rooting depth
    REAL(wp)              :: wsr_capacity         !< Water holding capacity of ground skin reservoir [m water equivalent]
    REAL(wp)              :: wsn_capacity         !< Water holding capacity of ground snow reservoir (max snow depth) [m water equivalent]
    REAL(wp)              :: soil_awc_prescribe, &
                             soil_theta_prescribe
    REAL(wp)              :: soil_sand            !< soil sand proportion - site specific from forcing info file
    REAL(wp)              :: soil_silt            !< soil silt proportion - site specific from forcing info file
    REAL(wp)              :: soil_clay            !< soil clay proportion - recalculated from: clay = 1.0 -sand -silt
    REAL(wp)              :: bulk_density    
  CONTAINS
    PROCEDURE :: Init => Init_sse_config
  END TYPE t_sse_config

  INTEGER, PARAMETER :: max_snow_layers = 20  ! consistent with JSBACH4
  INTEGER, PARAMETER :: max_soil_layers = 20  ! consistent with JSBACH4

  CHARACTER(len=*), PARAMETER :: modname = 'mo_sse_config_class'

CONTAINS

  !-----------------------------------------------------------------------------------------------------
  !> configuration routine of t_sse_config
  !! 
  !! currently it does only: read parameters from namelist
  !-----------------------------------------------------------------------------------------------------
  SUBROUTINE Init_sse_config(config)

    USE mo_namelist,           ONLY: open_nml, POSITIONED, position_nml, close_nml  ! read model options from file
    USE mo_jsb_grid_class,     ONLY: t_jsb_vgrid, new_vgrid
    USE mo_jsb_grid,           ONLY: Register_vgrid
    USE mo_jsb_grid_class,     ONLY: ZAXIS_GENERIC   ! in JSBACH4 used from mo_jsb_io    *slmtemp
    USE mo_sse_constants,      ONLY: snow_depth_min
    USE mo_jsb_math_constants, ONLY: eps8

    IMPLICIT NONE
    ! ---------------------------
    ! 0.1 InOut
    CLASS(t_sse_config), INTENT(inout) :: config    !< config type for sse
    ! ---------------------------
    ! 0.2 Local
    ! variables for reading from namlist, identical to variable-name in namelist
    LOGICAL                     :: active
    INTEGER                     :: nsoil_energy    , &
                                   nsoil_water
    REAL(wp)                    :: dz_energy(max_soil_layers), &                        ! vector of soil layer thicknesses 
                                   dz_water(max_soil_layers)
    REAL(wp)                    :: soil_layer_profile_ubound_estimate(max_soil_layers)  ! intial estimate of soil profile using the upper bound of each soil layer [m]
    REAL(wp), ALLOCATABLE       :: ubounds_soil_lay_energy(:), &                        ! layer upper bound (note! upper bound > lower bound)
                                   ubounds_soil_lay_water(:)
    REAL(wp)                    :: wsr_capacity    , &
                                   wsn_capacity    , &
                                   soil_depth   
    REAL(wp)                    :: soil_awc_prescribe, &
                                   soil_theta_prescribe
    REAL(wp)                    :: soil_sand       , &
                                   soil_silt       , &
                                   soil_clay
    REAL(wp)                    :: bulk_density
    INTEGER                     :: isoil   ! looping
    INTEGER                     :: i       ! looping over snow layers, consistent with jsb4
    CHARACTER(len=filename_max) :: ic_filename, bc_filename
    REAL(wp)                    :: k_soil_profile       ! for soil layer calculations
    REAL(wp)                    :: min_layer_depth      ! for soil layer calculations
    INTEGER                     :: nsnow
    REAL(wp)                    :: dz_snow(max_snow_layers)   ! width of each snow layer
    REAL(wp), ALLOCATABLE       :: depths(:), mids(:)         ! snow-layer vgrid init
    TYPE(t_jsb_vgrid), POINTER  :: vgrid_soil_sb              ! soil layers
    TYPE(t_jsb_vgrid), POINTER  :: snow_vgrid                 ! snow layers  <- name may be consistent with jsb4
    CHARACTER(len=*), PARAMETER :: routine = TRIM(modname)//':Init_sse_config'
    
    NAMELIST /sse_ctl/       & 
                          active                  , &
                          ic_filename             , &
                          bc_filename             , &
                          nsoil_energy            , &
                          nsoil_water             , &
                          dz_energy               , &
                          dz_water                , &
                          wsr_capacity            , &
                          wsn_capacity            , &
                          soil_depth              , &
                          soil_awc_prescribe      , &
                          soil_theta_prescribe    , &
                          soil_sand               , &
                          soil_silt               , &
                          soil_clay               , &
                          bulk_density            

    ! variables for reading model-options from namelist
    INTEGER :: nml_handler, nml_unit, istat

    ! variabales for soil layer calculations
    k_soil_profile  = 0.25_wp
    min_layer_depth = 0.065_wp

    CALL message(TRIM(routine), 'Starting sse configuration')

    ! Set defaults
    active                        = .TRUE.
    ic_filename                   = 'ic_land_sse.nc'
    bc_filename                   = 'bc_land_sse.nc'
    nsoil_energy                  = 5
    nsoil_water                   = 5
    dz_energy(1:5)                = (/0.065_wp,0.254_wp,0.913_wp,2.902_wp,5.700_wp/)
    dz_energy(6:max_soil_layers)  = 0._wp
    dz_water (1:5)                = (/0.065_wp,0.254_wp,0.913_wp,2.902_wp,5.700_wp/)
    dz_water (6:max_soil_layers)  = 0._wp 
    wsr_capacity                  = 2.E-4_wp
    wsn_capacity                  = 2.E-4_wp
    soil_depth                    = 5.7_wp 
    soil_awc_prescribe            = 300.0_wp
    soil_theta_prescribe          = 1.0_wp 
    soil_sand                     = 0.3_wp
    soil_silt                     = 0.4_wp
    soil_clay                     = 0.3_wp
    bulk_density                  = 1500.0_wp

    ! read the namelist
    ! &SSE_CTL
    nml_handler = open_nml(TRIM(config%namelist_filename))

    nml_unit = position_nml('SSE_CTL', nml_handler, STATUS=istat)
    IF (istat == POSITIONED) READ(nml_unit, sse_ctl)

    CALL close_nml(nml_handler)

    ! check initialisation of number of soil layers
    IF (nsoil_water .LT. 5 .OR. nsoil_water .GT. 20) & 
        CALL finish(routine,'Number of soil layers needs to be a number from the range 5 to 20.')
    IF (nsoil_water .NE. nsoil_energy) & 
        CALL finish(routine,'Init with different numbers of soil layers for energy and water not yet supported for QUINCY.')

    ! allocate upper-bound soil-layer vectors with dimensions defined in the namelist
    ALLOCATE(ubounds_soil_lay_energy(nsoil_energy))
    ALLOCATE(ubounds_soil_lay_water(nsoil_water))

    ! If not using 5 layers, estimate soil profile (i.e., dz == layer thicknesses) to mimick standard JSB4 profile
    !  assuming dz_water(:) = dz_energy(:)
    IF(nsoil_water > 5) THEN
      ! initial calculation of soil profile (for soil moisture calculations) calculating the upper bound of each layer
      DO isoil=1,nsoil_water
        soil_layer_profile_ubound_estimate(isoil) = min_layer_depth * & 
                                                    exp(k_soil_profile * isoil * REAL(max_soil_layers)/REAL(nsoil_water)) - & 
                                                    min_layer_depth
      ENDDO 
      ! final calculation of soil profile, calculating the layer thickness (dz(:)) and correcting for minimum soil-layer thickness
      dz_water(1) = min_layer_depth
      DO isoil=2,nsoil_water
        dz_water(isoil) = MAX(min_layer_depth, &
                               soil_layer_profile_ubound_estimate(isoil) - soil_layer_profile_ubound_estimate(isoil-1))
      ENDDO
      ! apply the same soil profile to soil profile for soil energy calculations
      dz_energy(:) = dz_water(:)
    ENDIF

    ! run check for soil depth namelist-definition; may not be less/equal zero or larger than covered by soil profile
    IF (soil_depth .LE. 0.0_wp) CALL finish('Init_sse_config','Initialization with non positive soil depth')
    IF (soil_depth .GT. SUM(dz_energy)) CALL finish('Init_sse_config','Initialization of soil depth larger than soil profile not possible')

    ! calc upper bound of soil layers (water and energy) from layer thickness
    ! note: upper bound is the larger value compared to lower bound of the same layer!
    ubounds_soil_lay_water(1)        = dz_water(1)
    DO isoil=2,nsoil_water
      ubounds_soil_lay_water(isoil)  = ubounds_soil_lay_water(isoil-1) + dz_water(isoil)
    END DO
    ubounds_soil_lay_energy(1)       = dz_energy(1)
    DO isoil=2,nsoil_energy
      ubounds_soil_lay_energy(isoil) = ubounds_soil_lay_energy(isoil-1) + dz_energy(isoil)
    END DO

    ! pass values as read from file
    config%active                    = active
    config%ic_filename               = ic_filename
    config%bc_filename               = bc_filename
    config%nsoil_energy              = nsoil_energy     ! depreciated feature, but needed for now, see ticket #521
    config%nsoil_water               = nsoil_water      ! depreciated feature, but needed for now, see ticket #521
    config%soil_awc_prescribe        = soil_awc_prescribe 
    config%soil_theta_prescribe      = soil_theta_prescribe 
    config%wsr_capacity              = wsr_capacity
    config%wsn_capacity              = wsn_capacity
    config%soil_depth                = soil_depth 
    config%soil_sand                 = soil_sand
    config%soil_silt                 = soil_silt
    !config%soil_clay                = not using the input value, as soil_clay must be calculated from soil_sand & soil_silt to guarantee: 1=sand+silt+clay
    config%soil_clay                 = 1.0_wp - soil_sand - soil_silt
    config%bulk_density              = bulk_density

    ! check for an error in clay content of soil
    IF(config%soil_clay < eps8) THEN
       WRITE(message_text,'(a)') 'invalid proportion of soil_clay (value < eps8)'
       CALL finish(routine, message_text)
    END IF

    !< Create vertical snow-layer axis (vgrid) - consistent with jsb4
    !! 
    !! these are "infrastructure" values \n
    !! layer 1 is the one at the ground \n
    !! snow_depth_snl(:,:) defines how much each of the layers if filled with snow
    !! 
    nsnow             = 5       ! by default 5 snow layer (in jsb4 it is a namelist value, which is actually not used)
    dz_snow(:)        = 0._wp   ! jsb4 default
    dz_snow(1:nsnow)  = 0.05_wp ! jsb4 default
    IF (dz_snow(1) < snow_depth_min)                                                                 &
      & CALL finish(TRIM(routine), 'Depth of first snow layer should be larger than snow_depth_min=' &
      &                            //real2string(snow_depth_min))
    ALLOCATE(depths(nsnow+1))
    ALLOCATE(mids(nsnow))
    depths(1) = 0._wp
    DO i=1,nsnow
      depths(i+1) = depths(i) + dz_snow(i)
    END DO
    mids(1:nsnow) = (depths(1:nsnow) + depths(2:nsnow+1)) / 2._wp

    snow_vgrid  => new_vgrid('snow_depth_energy', ZAXIS_GENERIC, nsnow,     &
      & levels  = mids                 (1:nsnow  ),                         &
      & lbounds = depths               (1:nsnow  ),                         &
      & ubounds = depths               (2:nsnow+1),                         &
      & units='m')
    CALL register_vgrid(snow_vgrid)
    WRITE(message_text, *) 'Snow levels in soil energy (upper) [m]: ', snow_vgrid%lbounds
    CALL message(TRIM(routine), message_text)
    WRITE(message_text, *) 'Snow levels in soil energy (mid)   [m]: ', snow_vgrid%levels
    CALL message(TRIM(routine), message_text)
    WRITE(message_text, *) 'Snow levels in soil energy (lower) [m]: ', snow_vgrid%ubounds
    CALL message(TRIM(routine), message_text)
    WRITE(message_text, *) 'Snow level depths in soil energy   [m]: ', snow_vgrid%dz
    CALL message(TRIM(routine), message_text)
    DEALLOCATE(depths, mids)

    !< Create vertical soil-layer axis (vgrid)
    !! 
    !! these are "infrastructure" values, ubounds and dz must be corrected/limited by site-specific bedrock depth "soil_depth" !
    !! 
    !! site-specific soil-layer thickness values are stored in soil_depth_sl (and calc in sse_init) \n
    !! the function new_vgrid() does: (a) check for 'dz(:) <= zero', and (b) calc levels & lbounds from ubounds and dz \n
    !! levels is defined as the depth at the center of the layer: levels(:) = 0.5_wp * (lbounds(:) + ubounds(:))
    vgrid_soil_sb => new_vgrid('soil_layer_sb', ZAXIS_GENERIC, nsoil_water, &
                      longname='Soil (water) layers from SSE_ QUINCY', &
                      units='m', &
                      ! levels=  , &      ! is calculated from ubounds and dz
                      ! lbounds=  , &     ! is calculated from ubounds and dz
                      ubounds=ubounds_soil_lay_water(:), & 
                      dz=dz_water(:))
    CALL register_vgrid(vgrid_soil_sb)

  END SUBROUTINE Init_sse_config

#endif
END MODULE mo_sse_config_class
